@@ -33,7 +33,6 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "xpad-periodic.h"
 #include "xpad-session-manager.h"
 #include "xpad-tray.h"
-#include "xpad-debug.h"
 
 #include <string.h>
 #include <sys/socket.h>
@@ -45,7 +44,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
 
-/* Seems that some systems (sun-sparc-solaris2.8 at least), need the following three #defines. 
+/* Seems that some systems (sun-sparc-solaris2.8 at least), need the following three #defines.
    These were provided by Alan Mizrahi <alan@cesma.usb.ve>.
 */
 #ifndef PF_LOCAL
@@ -69,6 +68,7 @@ static gboolean option_show;
 static gboolean option_toggle;
 static gboolean option_version;
 static gboolean option_quit;
+static gboolean shutdown_in_progress;
 static gchar **option_files;
 static gchar *option_smid;
 static gchar *config_dir;
@@ -85,8 +85,8 @@ static gboolean		process_remote_args         (gint *argc, gchar **argv[], gboole
 
 static gboolean		config_dir_exists           (void);
 static gchar		*make_config_dir            (void);
-static void			register_stock_icons        (void);
-static gint			xpad_app_load_pads          (void);
+static void		register_stock_icons        (void);
+static gint		xpad_app_load_pads          (void);
 static gboolean		xpad_app_quit_if_no_pads    (XpadPadGroup *group);
 static gboolean		xpad_app_first_idle_check   (XpadPadGroup *group);
 static gboolean		xpad_app_pass_args          (void);
@@ -97,9 +97,7 @@ xpad_app_init (int argc, char **argv)
 {
 	gboolean first_time;
 	gboolean have_gtk;
-
-	xpad_debug_init();
-	xpad_debug_message (DEBUG_APP, "Startup");
+	shutdown_in_progress = FALSE;
 
 	/* Set up support different languages */
 #ifdef ENABLE_NLS
@@ -184,18 +182,13 @@ xpad_app_init (int argc, char **argv)
 		}
 	}
 
-	/* Since all pads have been loaded, reprocess the show/hide/toggle option for all pads */
-	if (have_gtk && (option_show))
-		xpad_pad_group_show_all (pad_group);
-	if (have_gtk && (option_hide))
-		xpad_pad_group_close_all (pad_group);
-	if (have_gtk && option_toggle)
-		xpad_pad_group_toggle_hide (pad_group);
-
 	g_idle_add ((GSourceFunc)xpad_app_first_idle_check, pad_group);
 
-	if (first_time)
+	if (first_time) {
+		gboolean autostart_xpad = TRUE;
+		g_object_set (settings, "autostart-xpad", &autostart_xpad, NULL);
 		show_help ();
+	}
 
 	g_free (server_filename);
 	server_filename = NULL;
@@ -253,6 +246,11 @@ xpad_app_get_pad_group (void)
 void
 xpad_app_quit (void)
 {
+	if (shutdown_in_progress)
+		return;
+
+	shutdown_in_progress = TRUE;
+
 	/* Free the memory used by the pads belonging to this group */
 	xpad_pad_group_destroy_pads (pad_group);
 
@@ -384,8 +382,8 @@ xpad_app_alert_dialog (GtkWindow *parent, const gchar *icon_name, const gchar *p
 	gtk_container_add (GTK_CONTAINER (hbox), image);
 	gtk_container_add (GTK_CONTAINER (hbox), label);
 
-	gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0);
-	gtk_misc_set_alignment (GTK_MISC (label), 0.5, 0);
+	gtk_widget_set_halign (image, GTK_ALIGN_CENTER);
+	gtk_widget_set_halign (label, GTK_ALIGN_CENTER);
 	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
 	gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
 	gtk_container_set_border_width (GTK_CONTAINER (dialog), 6);
@@ -483,16 +481,19 @@ xpad_app_load_pads (void)
 	while ((name = g_dir_read_name (dir)))
 	{
 		/* if it's an info file, but not a backup info file... */
-		if (!strncmp (name, "info-", 5) &&
-		    name[strlen (name) - 1] != '~')
+		if (!strncmp (name, "info-", 5) && name[strlen (name) - 1] != '~')
 		{
 			gboolean show = TRUE;
 			GtkWidget *pad = xpad_pad_new_with_info (pad_group, settings, name, &show);
+			/*
+			 * show = refers to the hidden variable in the info file; this can be different for each pad; show = !hidden.
+			 * option_show = command line parameter to show all the pads
+			 * option_hide = command line parameter to hide all the pads
+			*/
 			if ((show || option_show) && !option_hide)
 				gtk_widget_show (pad);
-		else if (show) /* pad thought it would show, we should save that it didn't */
-			xpad_pad_save_info (XPAD_PAD (pad));
-			
+			else if (show) /* pad thought it would show, we should save that it didn't */
+				xpad_pad_save_info (XPAD_PAD (pad));
 			opened ++;
 		}
 	}
@@ -652,7 +653,7 @@ xpad_app_read_from_proc_file (void)
 		v[1] = "--new";
 
 		process_remote_args (&c, &v, TRUE, settings);
-		
+
 		g_free (v);
 	}
 
@@ -690,7 +691,7 @@ xpad_app_open_proc_file (void)
 
 	/* create the socket */
 	server_fd = socket (PF_LOCAL, SOCK_STREAM, 0);
-	bzero (&master, sizeof (master)); 
+	bzero (&master, sizeof (master));
 	master.sun_family = AF_LOCAL;
 	strcpy (master.sun_path, server_filename);
 
@@ -756,7 +757,7 @@ xpad_app_pass_args (void)
 		do
 		{
 			bytesRead = read (client_fd, buf, 128);
-			
+
 			if (bytesRead < 0)
 			{
 			  goto done;
@@ -768,7 +769,7 @@ xpad_app_pass_args (void)
 		while (bytesRead > 0);
 	}
 	while (bytesRead > 0);
-	
+
 done:
 	close (client_fd);
 
@@ -811,7 +812,7 @@ process_local_args (gint *argc, gchar **argv[])
 	option_version = FALSE;
 	option_nonew = FALSE;
 
-	/* We make copies of argc and argv because we actually don't want the 
+	/* We make copies of argc and argv because we actually don't want the
 	   behavior of g_option_context_parse() that removes entries from the
 	   array. */
 	argc_copy = *argc;
@@ -865,7 +866,7 @@ process_remote_args (gint *argc, gchar **argv[], gboolean have_gtk, XpadSettings
 	{
 		if (have_gtk && option_smid)
 			xpad_session_manager_set_id (option_smid);
-		
+
 		if (!option_new)
 			g_object_get (settings, "autostart-new-pad", &option_new, NULL);
 
@@ -875,6 +876,13 @@ process_remote_args (gint *argc, gchar **argv[], gboolean have_gtk, XpadSettings
 			gtk_widget_show (pad);
 		}
 
+		if (have_gtk && (option_show))
+			xpad_pad_group_show_all (pad_group);
+		if (have_gtk && (option_hide))
+			xpad_pad_group_close_all (pad_group);
+		if (have_gtk && option_toggle)
+			xpad_pad_group_toggle_hide (pad_group);
+
 		if (!option_hide && !option_show) {
 			guint display_pads;
 			g_object_get (xpad_settings, "autostart-display-pads", &display_pads, NULL);
