@@ -5,7 +5,7 @@ Copyright (c) 2009 Paul Ivanov
 Copyright (c) 2011 Sergei Riaguzov
 Copyright (c) 2011 Dennis Hilmar
 Copyright (c) 2011 OBATA Akio
-Copyright (c) 2013-2014 Arthur Borsboom
+Copyright (c) 2013-2015 Arthur Borsboom
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -120,6 +120,8 @@ static void xpad_pad_notify_has_scrollbar (XpadPad *pad);
 static void xpad_pad_notify_has_decorations (XpadPad *pad);
 static void xpad_pad_notify_has_toolbar (XpadPad *pad);
 static void xpad_pad_notify_autohide_toolbar (XpadPad *pad);
+static void xpad_pad_notify_hide_from_taskbar (XpadPad *pad);
+static void xpad_pad_notify_hide_from_task_switcher (XpadPad *pad);
 static void xpad_pad_hide_toolbar (XpadPad *pad);
 static void xpad_pad_show_toolbar (XpadPad *pad);
 static void xpad_pad_popup (XpadPad *pad, GdkEventButton *event);
@@ -142,15 +144,19 @@ static gboolean xpad_pad_enter_notify_event (GtkWidget *pad, GdkEventCrossing *e
 GtkWidget *
 xpad_pad_new (XpadPadGroup *group, XpadSettings *settings)
 {
-	return GTK_WIDGET (g_object_new (XPAD_TYPE_PAD, "group", group, "settings", settings, NULL));
+	GtkWidget *pad = GTK_WIDGET (g_object_new (XPAD_TYPE_PAD, "group", group, "settings", settings, NULL));
+
+	xpad_pad_save_info_delayed (XPAD_PAD (pad));
+
+	return pad;
 }
 
 /* Create a new pad based on the provided info-xxxxx file from the config directory and return this pad */
 GtkWidget *
 xpad_pad_new_with_info (XpadPadGroup *group, XpadSettings *settings, const gchar *info_filename, gboolean *show)
 {
-	GtkWidget *pad = xpad_pad_new (group, settings);
-	
+	GtkWidget *pad = GTK_WIDGET (g_object_new (XPAD_TYPE_PAD, "group", group, "settings", settings, NULL));
+
 	XPAD_PAD (pad)->priv->infoname = g_strdup (info_filename);
 	xpad_pad_load_info (XPAD_PAD (pad), show);
 	xpad_pad_load_content (XPAD_PAD (pad));
@@ -301,7 +307,7 @@ static void xpad_pad_constructed (GObject *object)
 {
 	XpadPad *pad = XPAD_PAD (object);
 
-	gboolean decorations;
+	gboolean decorations, hide_from_taskbar, hide_from_task_switcher;
 	GtkBox *vbox;
 
 	g_object_get (pad->priv->settings,
@@ -338,11 +344,13 @@ static void xpad_pad_constructed (GObject *object)
 	gtk_container_child_set (GTK_CONTAINER (vbox), pad->priv->toolbar, "expand", FALSE, NULL);
 
 	g_object_get (pad->priv->settings, "has-decorations", &decorations, NULL);
+	g_object_get (pad->priv->settings, "hide-from-taskbar", &hide_from_taskbar, NULL);
+	g_object_get (pad->priv->settings, "hide-from-task-switcher", &hide_from_task_switcher, NULL);
 	gtk_window_set_decorated (pad_window, decorations);
 	gtk_window_set_default_size (pad_window, (gint) pad->priv->width, (gint) pad->priv->height);
 	gtk_window_set_gravity (pad_window, GDK_GRAVITY_STATIC); /* static gravity makes saving pad x,y work */
-	gtk_window_set_skip_pager_hint (pad_window, decorations);
-	gtk_window_set_skip_taskbar_hint (pad_window, !decorations);
+	gtk_window_set_skip_taskbar_hint (pad_window, hide_from_taskbar);
+	gtk_window_set_skip_pager_hint (pad_window, hide_from_task_switcher);
 	gtk_window_set_position (pad_window, GTK_WIN_POS_MOUSE);
 
 	g_object_set (G_OBJECT (pad), "child", vbox, NULL);
@@ -382,6 +390,8 @@ static void xpad_pad_constructed (GObject *object)
 	g_signal_connect (pad, "enter-notify-event", G_CALLBACK (xpad_pad_enter_notify_event), NULL);
 	g_signal_connect (pad, "leave-notify-event", G_CALLBACK (xpad_pad_leave_notify_event), NULL);
 
+	g_signal_connect_swapped (pad->priv->settings, "notify::hide-from-taskbar", G_CALLBACK (xpad_pad_notify_hide_from_taskbar), pad);
+	g_signal_connect_swapped (pad->priv->settings, "notify::hide-from-task-switcher", G_CALLBACK (xpad_pad_notify_hide_from_task_switcher), pad);
 	g_signal_connect_swapped (pad->priv->settings, "notify::has-decorations", G_CALLBACK (xpad_pad_notify_has_decorations), pad);
 	g_signal_connect_swapped (pad->priv->settings, "notify::has-toolbar", G_CALLBACK (xpad_pad_notify_has_toolbar), pad);
 	g_signal_connect_swapped (pad->priv->settings, "notify::autohide-toolbar", G_CALLBACK (xpad_pad_notify_autohide_toolbar), pad);
@@ -459,7 +469,7 @@ xpad_pad_finalize (GObject *object)
 
 	g_free (pad->priv->infoname);
 	g_free (pad->priv->contentname);
-	
+
 	G_OBJECT_CLASS (xpad_pad_parent_class)->finalize (object);
 }
 
@@ -482,6 +492,10 @@ xpad_pad_show (XpadPad *pad)
 		gtk_window_stick (GTK_WINDOW (pad));
 	else
 		gtk_window_unstick (GTK_WINDOW (pad));
+
+	/* Show the pad and set the cursor into the pad */
+        gtk_window_present (GTK_WINDOW (pad));
+        gtk_widget_grab_focus (GTK_WIDGET (pad->priv->textview));
 }
 
 static gboolean toolbar_timeout (XpadPad *pad)
@@ -510,11 +524,9 @@ xpad_pad_notify_has_decorations (XpadPad *pad)
 
 	/*
 	 *  There are two modes of operation:  a normal mode and a 'stealth' mode.
-	 *  If decorations are disabled, we also don't show up in the taskbar or pager. 
+	 *  If decorations are disabled, we also don't show up in the taskbar or pager.
 	 */
 	gtk_window_set_decorated (pad_window, decorations);
-	gtk_window_set_skip_taskbar_hint (pad_window, !decorations);
-	gtk_window_set_skip_pager_hint (pad_window, !decorations);
 
 	/*
 	 * reshow_with_initial_size() seems to set the window back to a never-shown state.
@@ -527,6 +539,22 @@ xpad_pad_notify_has_decorations (XpadPad *pad)
 	gtk_widget_show (pad_widget);
 }
 
+static void
+xpad_pad_notify_hide_from_taskbar (XpadPad *pad)
+{
+	gboolean hide;
+	g_object_get (pad->priv->settings, "hide-from-taskbar", &hide, NULL);
+	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (pad), hide);
+}
+
+static void
+xpad_pad_notify_hide_from_task_switcher (XpadPad *pad)
+{
+	gboolean hide;
+	g_object_get (pad->priv->settings, "hide-from-task-switcher", &hide, NULL);
+	gtk_window_set_skip_pager_hint (GTK_WINDOW (pad), hide);
+}
+
 static void
 xpad_pad_notify_has_toolbar (XpadPad *pad)
 {
@@ -662,7 +690,7 @@ xpad_pad_hide_toolbar (XpadPad *pad)
 		if (gtk_widget_get_window (pad_widget))
 			gdk_window_freeze_updates (gtk_widget_get_window (pad_widget));
 		gtk_widget_hide (pad->priv->toolbar);
-		
+
 		if (pad->priv->toolbar_expanded ||
 			 (pad->priv->toolbar_pad_resized && xpad_pad_text_and_toolbar_height (pad) >= pad->priv->height))
 		{
@@ -817,11 +845,11 @@ should_confirm_delete (XpadPad *pad)
 	g_object_get (pad->priv->settings, "confirm-destroy", &confirm, NULL);
 	if (!confirm)
 		return FALSE;
-	
+
 	buffer = GTK_SOURCE_BUFFER (gtk_text_view_get_buffer (GTK_TEXT_VIEW (pad->priv->textview)));
 	gtk_text_buffer_get_bounds (GTK_TEXT_BUFFER (buffer), &s, &e);
 	content = gtk_text_buffer_get_text (GTK_TEXT_BUFFER (buffer), &s, &e, FALSE);
-	
+
 	confirm = strcmp (g_strstrip (content), "") != 0;
 
 	g_free (content);
@@ -844,7 +872,7 @@ xpad_pad_delete (XpadPad *pad)
 		gint response;
 
 		dialog = xpad_app_alert_dialog (GTK_WINDOW (pad), "dialog-warning", _("Delete this pad?"), _("All text of this pad will be irrevocably lost."));
-		
+
 		if (!dialog)
 			return;
 
@@ -864,6 +892,13 @@ xpad_pad_delete (XpadPad *pad)
 	if (pad->priv->contentname)
 		fio_remove_file (pad->priv->contentname);
 
+	/* Before deleting the current pad, find and set the focus to another pad (if any) */
+	GSList *nextPad = g_slist_nth (xpad_pad_group_get_pads(pad->priv->group), 0);
+	if (nextPad->data == pad)
+		nextPad = g_slist_next (nextPad);
+	if (nextPad)
+        	xpad_pad_show (nextPad->data);
+
 	/* Remove the pad from the group and destroy it. */
 	gtk_widget_destroy (GTK_WIDGET (pad));
 }
@@ -1029,7 +1064,7 @@ xpad_pad_toolbar_size_allocate (XpadPad *pad, GtkAllocation *event)
 static gboolean
 xpad_pad_configure_event (XpadPad *pad, GdkEventConfigure *event)
 {
-	if (!gtk_widget_get_visible (GTK_WIDGET(pad)))
+	if (!gtk_widget_get_visible (GTK_WIDGET (pad)))
 		return FALSE;
 
 	int eWidth = event->width;
@@ -1037,29 +1072,30 @@ xpad_pad_configure_event (XpadPad *pad, GdkEventConfigure *event)
 
 	/* safe cast from gint to guint */
 	if (eWidth >= 0 && eHeight >=0 ) {
-		if (pad->priv->width != (guint) eWidth || pad->priv->height != (guint) eHeight)
+		if (pad->priv->width != (guint) eWidth || pad->priv->height != (guint) eHeight) {
 			pad->priv->toolbar_pad_resized = TRUE;
-
-		pad->priv->width = (guint) event->width;
-		pad->priv->height = (guint) event->height;
+			pad->priv->width = (guint) eWidth;
+			pad->priv->height = (guint) eHeight;
+			pad->priv->unsaved_info = TRUE;
+		}
 	}
 	else {
 		g_warning("There is a problem in the program Xpad. In function 'xpad_pad_configure_event' the variable 'event->width' or 'event->height' is not a postive number. Please send a bugreport to https://bugs.launchpad.net/xpad/+filebug to help improve Xpad.");
 	}
 
-	pad->priv->x = event->x;
-	pad->priv->y = event->y;
-	pad->priv->location_valid = TRUE;
-
-	xpad_pad_save_info_delayed(pad);
+	if (pad->priv->x != event->x || pad->priv->y != event->y) {
+		pad->priv->x = event->x;
+		pad->priv->y = event->y;
+		pad->priv->location_valid = TRUE;
+		pad->priv->unsaved_info = TRUE;
+	}
 
 	/*
 	 * Sometimes when moving, if the toolbar tries to hide itself,
 	 * the window manager will not resize it correctly.  So, we make
 	 * sure not to end the timeout while moving.
 	 */
-	if (pad->priv->toolbar_timeout)
-	{
+	if (pad->priv->toolbar_timeout) {
 		g_source_remove (pad->priv->toolbar_timeout);
 		pad->priv->toolbar_timeout = g_timeout_add (1000, (GSourceFunc) toolbar_timeout, pad);
 	}
@@ -1176,7 +1212,7 @@ xpad_pad_save_content (XpadPad *pad)
 {
 	g_return_if_fail (pad);
 
-	gchar *content;
+	gchar *content = NULL;
 	XpadTextBuffer *buffer;
 
 	if (!pad->priv->unsaved_content)
@@ -1618,7 +1654,7 @@ menu_prep_popup_no_highlight (XpadPad *pad, GtkWidget *uppermenu)
 	{
 		gint n = 1;
 		gchar *key;
-		
+
 		/* Remove old notes */
 		item = g_object_get_data (G_OBJECT (menu), "notes-sep");
 		while (item)
@@ -1655,7 +1691,7 @@ void xpad_pad_append_pad_titles_to_menu (GtkWidget *menu)
 		gchar *key;
 
 		key = g_strdup_printf ("notes-%i", n);
-		tmp_title = g_strdup (gtk_window_get_title (GTK_WINDOW (l->data)));
+		tmp_title = g_strndup (gtk_window_get_title (GTK_WINDOW (l->data)), 20);
 		str_replace_tokens (&tmp_title, '_', "__");
 		if (n < 10)
 			title = g_strdup_printf ("_%i. %s", n, tmp_title);
