@@ -2,6 +2,7 @@
 
 Copyright (c) 2001-2007 Michael Terry
 Copyright (c) 2010 Sergei Riaguzov
+Copyright (c) 2013-2014 Arthur Borsboom
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -19,31 +20,34 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 */
 
-
 #include "../config.h"
-#include <stdlib.h>
-#include <glib.h>
+
 #include "xpad-undo.h"
 #include "xpad-text-buffer.h"
 
+#include <stdlib.h>
+
+#include <glib.h>
+
 struct XpadUndoPrivate
 {
 	XpadTextBuffer *buffer;
-	/* We always redo the next element in the list but undo the current one.
-		We insert this guard with NULL data in the beginning to ease coding all of this */
+	/*
+	 * We always redo the next element in the list but undo the current one.
+	 * We insert this guard with NULL data in the beginning to ease coding all of this.
+	 */
 	GList *history_start;
 	GList *history_curr;
 	guint user_action;
 	gboolean frozen;
 };
 
-G_DEFINE_TYPE_WITH_PRIVATE(XpadUndo, xpad_undo, G_TYPE_OBJECT)
+G_DEFINE_TYPE_WITH_PRIVATE (XpadUndo, xpad_undo, G_TYPE_OBJECT)
 
-static GObject* xpad_undo_constructor(GType gtype, guint n_properties, GObjectConstructParam *properties);
-static void xpad_undo_dispose (GObject *object);
-static void xpad_undo_finalize (GObject *object);
 static void xpad_undo_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
 static void xpad_undo_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
+static void xpad_undo_constructed (GObject *object);
+static void xpad_undo_finalize (GObject *object);
 
 enum UserActionType
 {
@@ -67,10 +71,10 @@ typedef struct
 static GList* xpad_undo_remove_action_elem (GList *curr);
 static void xpad_undo_clear_redo_history (XpadUndo *undo);
 static void xpad_undo_clear_history (XpadUndo *undo);
-static void xpad_undo_begin_user_action (GtkTextBuffer *buffer, XpadUndo *undo);
-static void xpad_undo_end_user_action (GtkTextBuffer *buffer, XpadUndo *undo);
-static void xpad_undo_insert_text (GtkTextBuffer *buffer, GtkTextIter *location, gchar *text, gint len, XpadUndo *undo);
-static void xpad_undo_delete_range (GtkTextBuffer *buffer, GtkTextIter *start, GtkTextIter *end, XpadUndo *undo);
+static void xpad_undo_begin_user_action (GtkSourceBuffer *buffer, XpadUndo *undo);
+static void xpad_undo_end_user_action (GtkSourceBuffer *buffer, XpadUndo *undo);
+static void xpad_undo_insert_text (GtkSourceBuffer *buffer, GtkTextIter *location, gchar *text, gint len, XpadUndo *undo);
+static void xpad_undo_delete_range (GtkSourceBuffer *buffer, GtkTextIter *start, GtkTextIter *end, XpadUndo *undo);
 
 enum
 {
@@ -90,16 +94,15 @@ xpad_undo_class_init (XpadUndoClass *klass)
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
-	gobject_class->dispose = xpad_undo_dispose;
-	gobject_class->finalize = xpad_undo_finalize;
 	gobject_class->set_property = xpad_undo_set_property;
 	gobject_class->get_property = xpad_undo_get_property;
-	gobject_class->constructor = xpad_undo_constructor;
+	gobject_class->constructed = xpad_undo_constructed;
+	gobject_class->finalize = xpad_undo_finalize;
 
 	g_object_class_install_property (gobject_class,
 					 PROP_BUFFER,
 					 g_param_spec_pointer ("buffer",
-									"Pad Buffer",
+									"Pad buffer",
 									"Pad buffer connected to this undo",
 									G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 }
@@ -107,7 +110,7 @@ xpad_undo_class_init (XpadUndoClass *klass)
 static void
 xpad_undo_init (XpadUndo *undo)
 {
-	undo->priv = xpad_undo_get_instance_private(undo);
+	undo->priv = xpad_undo_get_instance_private (undo);
 
 	undo->priv->buffer = NULL;
 	undo->priv->history_start = g_list_append (NULL, NULL);
@@ -116,54 +119,14 @@ xpad_undo_init (XpadUndo *undo)
 	undo->priv->frozen = FALSE;
 }
 
-static GObject*
-xpad_undo_constructor(GType gtype, guint n_properties, GObjectConstructParam *properties)
-{
-	GObjectClass *parent_class = G_OBJECT_CLASS (xpad_undo_parent_class);
-	GObject *obj = parent_class->constructor (gtype, n_properties, properties);
-
-	XpadUndo *undo = XPAD_UNDO (obj);
-
-	/* Assert user passed buffer and pad as construct parameter */
-	guint i;
-	gboolean found_buffer = FALSE;
-	for (i = 0; i < n_properties; i++)
-	{
-		if (!g_strcmp0 (properties[i].pspec->name, "buffer") &&
-			G_VALUE_HOLDS_POINTER (properties[i].value) &&
-			G_IS_OBJECT (g_value_get_pointer (properties[i].value)))
-		{
-			found_buffer = TRUE;
-		}
-	}
-
-	if (!found_buffer)
-	{
-		undo->priv->buffer = NULL;
-		g_warning ("XpadTextBuffer is not passed to XpadUndo constructor, undo will not work!\n");
-	}
-	else
-	{
-		/* Set up signals */
-		g_signal_connect (G_OBJECT (undo->priv->buffer), "insert-text", G_CALLBACK (xpad_undo_insert_text), undo);
-		g_signal_connect (G_OBJECT (undo->priv->buffer), "delete-range", G_CALLBACK (xpad_undo_delete_range), undo);
-		g_signal_connect (G_OBJECT (undo->priv->buffer), "begin-user-action", G_CALLBACK (xpad_undo_begin_user_action), undo);
-		g_signal_connect (G_OBJECT (undo->priv->buffer), "end-user-action", G_CALLBACK (xpad_undo_end_user_action), undo);
-	}
-
-	return obj;
-}
-
-static void
-xpad_undo_dispose (GObject *object)
+static void xpad_undo_constructed (GObject *object)
 {
 	XpadUndo *undo = XPAD_UNDO (object);
 
-	if (undo->priv->buffer) {
-		undo->priv->buffer = NULL;
-	}
-
-	G_OBJECT_CLASS (xpad_undo_parent_class)->dispose (object);
+	g_signal_connect (G_OBJECT (undo->priv->buffer), "insert-text", G_CALLBACK (xpad_undo_insert_text), undo);
+	g_signal_connect (G_OBJECT (undo->priv->buffer), "delete-range", G_CALLBACK (xpad_undo_delete_range), undo);
+	g_signal_connect (G_OBJECT (undo->priv->buffer), "begin-user-action", G_CALLBACK (xpad_undo_begin_user_action), undo);
+	g_signal_connect (G_OBJECT (undo->priv->buffer), "end-user-action", G_CALLBACK (xpad_undo_end_user_action), undo);
 }
 
 static void
@@ -175,7 +138,7 @@ xpad_undo_finalize (GObject *object)
 	xpad_undo_clear_history (undo);
 
 	/* remove left guard */
-	g_list_free(undo->priv->history_start);
+	g_list_free (undo->priv->history_start);
 
 	G_OBJECT_CLASS (xpad_undo_parent_class)->finalize (object);
 }
@@ -188,19 +151,14 @@ xpad_undo_set_property (GObject *object, guint prop_id, const GValue *value, GPa
 	switch (prop_id)
 	{
 	case PROP_BUFFER:
-		if (undo->priv->buffer && G_IS_OBJECT (undo->priv->buffer))
-			g_object_unref (undo->priv->buffer);
-		if (G_VALUE_HOLDS_POINTER (value) && G_IS_OBJECT (g_value_get_pointer (value)))
-		{
-			undo->priv->buffer = g_value_get_pointer (value);
-			g_object_ref (undo->priv->buffer);
-		}
+		undo->priv->buffer = g_value_get_pointer (value);
+		g_object_ref (undo->priv->buffer);
 		break;
 
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
-	 }
+	}
 }
 
 static void
@@ -221,14 +179,20 @@ xpad_undo_get_property (GObject *object, guint prop_id, GValue *value, GParamSpe
 }
 
 static void
-xpad_undo_begin_user_action (GtkTextBuffer *buffer, XpadUndo *undo)
+xpad_undo_begin_user_action (GtkSourceBuffer *buffer, XpadUndo *undo)
 {
+	/* A dirty way to silence the compiler for these unused variables. */
+	(void) buffer;
+
 	undo->priv->user_action++;
 }
 
 static void
-xpad_undo_end_user_action (GtkTextBuffer *buffer, XpadUndo *undo)
+xpad_undo_end_user_action (GtkSourceBuffer *buffer, XpadUndo *undo)
 {
+	/* A dirty way to silence the compiler for these unused variables. */
+	(void) buffer;
+
 	if (undo->priv->user_action > 0)
 		undo->priv->user_action--;
 }
@@ -255,8 +219,10 @@ xpad_undo_remove_action_elem (GList *curr)
 	return curr;
 }
 
-/* Redo is impossible after text insertion/deletion, only right after Undo (another Redo),
-	so we have to remove every history entry after the current one if there is any */
+/*
+ * Redo is impossible after text insertion/deletion, only right after Undo (another Redo),
+ * so we have to remove every history entry after the current one if there is any
+ */
 static void
 xpad_undo_clear_redo_history (XpadUndo *undo)
 {
@@ -273,20 +239,35 @@ xpad_undo_clear_history (XpadUndo *undo)
 }
 
 static void
-xpad_undo_insert_text (GtkTextBuffer *buffer, GtkTextIter *location, gchar *text, gint len, XpadUndo *undo)
+xpad_undo_insert_text (GtkSourceBuffer *buffer, GtkTextIter *location, gchar *text, gint len, XpadUndo *undo)
 {
+	/* A dirty way to silence the compiler for these unused variables. */
+	(void) buffer;
+
 	if (undo->priv->frozen)
 		return;
 
 	if (undo->priv->user_action)
 	{
+		XpadPad *pad;
+		gint n_utf8_chars = 0;
+		glong string_length = 0;
+
 		xpad_undo_clear_redo_history (undo);
 
 		gint pos = gtk_text_iter_get_offset (location);
-		gint n_utf8_chars = g_utf8_strlen (text, len);
 
-		/* Merge similar actions. This is how Undo works in most editors, if there is a series of
-			1-letter insertions - they are merge for Undo */
+		/* safe cast */
+		string_length = g_utf8_strlen (text, len);
+		if (string_length <= UINT_MAX)
+			n_utf8_chars = (gint) string_length;
+		else
+			g_warning("Casting problem in undo insert text function. Please send a bugreport.");
+
+		/*
+		 * Merge similar actions. This is how Undo works in most editors, if there
+		 * is a series of 1-letter insertions - they are merge for Undo
+		 */
 		if (undo->priv->history_curr->data)
 		{
 			UserAction *prev_action = undo->priv->history_curr->data;
@@ -294,13 +275,13 @@ xpad_undo_insert_text (GtkTextBuffer *buffer, GtkTextIter *location, gchar *text
 			if (prev_action->action_type == USER_ACTION_INSERT_TEXT)
 			{
 				/* series of 1-letter insertions */
-				if (n_utf8_chars == 1 // this is a 1-letter insertion
-					&& pos == prev_action->end // placed right after the previous text
-					&& (prev_action->n_utf8_chars == 1 || prev_action->merged)) // with which we should merge
+				if (n_utf8_chars == 1 /* this is a 1-letter insertion */
+					&& pos == prev_action->end /* placed right after the previous text */
+					&& (prev_action->n_utf8_chars == 1 || prev_action->merged)) /* with which we should merge */
 				{
 					/* if there was a space stop merging unless that was a series of spaces */
-					if ((!g_unichar_isspace (prev_action->text[0]) && !g_ascii_isspace (text[0])) ||
-						(g_unichar_isspace (prev_action->text[0]) && g_unichar_isspace (text[0])))
+					if ((!g_unichar_isspace ((gunichar) prev_action->text[0]) && !g_ascii_isspace ((gunichar) text[0])) ||
+						(g_unichar_isspace ((gunichar) prev_action->text[0]) && g_unichar_isspace ((gunichar) text[0])))
 					{
 						gchar *joined_str = g_strjoin (NULL, prev_action->text, text, NULL);
 						g_free (prev_action->text);
@@ -324,32 +305,51 @@ xpad_undo_insert_text (GtkTextBuffer *buffer, GtkTextIter *location, gchar *text
 		action->n_utf8_chars = n_utf8_chars;
 		action->merged = FALSE;
 
-		/* since each operation clears redo we know that there
-			is nothing after history_curr at this point so we
-			insert right after it. history_start won't change
-			since it is a left guard - not NULL */
-		GList *dummy_start = g_list_append (undo->priv->history_curr, action); // supress warning, we have left guard for start
+		/*
+		 * Since each operation clears redo we know that there is nothing
+		 * after history_curr at this point so we insert right after it.
+		 * history_start won't change since it is a left guard - not NULL
+		 */
+		GList *dummy_start = g_list_append (undo->priv->history_curr, action);
+
+		/* A dirty way to silence the compiler for these unused variables. */
+		(void) dummy_start;
+
 		undo->priv->history_curr = g_list_next (undo->priv->history_curr);
 
-		xpad_pad_notify_undo_redo_changed (xpad_text_buffer_get_pad (undo->priv->buffer));
+		g_object_get (undo->priv->buffer, "pad", &pad, NULL);
+		xpad_pad_notify_undo_redo_changed (pad);
 	}
 }
 
 static void
-xpad_undo_delete_range (GtkTextBuffer *buffer, GtkTextIter *start, GtkTextIter *end, XpadUndo *undo)
+xpad_undo_delete_range (GtkSourceBuffer *buffer, GtkTextIter *start, GtkTextIter *end, XpadUndo *undo)
 {
+	/* A dirty way to silence the compiler for these unused variables. */
+	(void) buffer;
+
 	if (undo->priv->frozen)
 		return;
 
 	if (undo->priv->user_action)
 	{
+		XpadPad *pad;
+		gint n_utf8_chars = 0;
+		glong string_length = 0;
+
 		xpad_undo_clear_redo_history (undo);
 
 		gchar *text = gtk_text_iter_get_text (start, end);
 		gint start_offset = gtk_text_iter_get_offset (start);
 		gint end_offset = gtk_text_iter_get_offset (end);
 		gint len = abs (end_offset - start_offset);
-		gint n_utf8_chars = g_utf8_strlen (text, len);
+
+		/* safe cast */
+		string_length = g_utf8_strlen (text, len);
+		if (string_length <= UINT_MAX)
+			n_utf8_chars = (gint) string_length;
+		else
+			g_warning("Casting problem in undo delete range function. Please send a bugreport.");
 
 		UserAction *action = g_new (UserAction, 1);
 		action->action_type = USER_ACTION_DELETE_RANGE;
@@ -360,10 +360,15 @@ xpad_undo_delete_range (GtkTextBuffer *buffer, GtkTextIter *start, GtkTextIter *
 		action->n_utf8_chars = n_utf8_chars;
 		action->merged = FALSE;
 
-		GList *dummy_start = g_list_append (undo->priv->history_curr, action); // supress warning, we have left guard for start
+		GList *dummy_start = g_list_append (undo->priv->history_curr, action);
+
+		/* A dirty way to silence the compiler for these unused variables. */
+		(void) dummy_start;
+
 		undo->priv->history_curr = g_list_next (undo->priv->history_curr);
 
-		xpad_pad_notify_undo_redo_changed (xpad_text_buffer_get_pad (undo->priv->buffer));
+		g_object_get (undo->priv->buffer, "pad", &pad, NULL);
+		xpad_pad_notify_undo_redo_changed (pad);
 	}
 }
 
@@ -373,6 +378,8 @@ xpad_undo_apply_tag (XpadUndo *undo, const gchar *name, GtkTextIter *start, GtkT
 	if (undo->priv->frozen)
 		return;
 
+	XpadPad *pad;
+
 	xpad_undo_clear_redo_history (undo);
 
 	gint start_offset = gtk_text_iter_get_offset (start);
@@ -382,13 +389,18 @@ xpad_undo_apply_tag (XpadUndo *undo, const gchar *name, GtkTextIter *start, GtkT
 	action->action_type = USER_ACTION_APPLY_TAG;
 	action->text = g_strdup (name);
 	action->start = start_offset;
-	action->end = end_offset;	
+	action->end = end_offset;
 	action->merged = FALSE;
 
-	GList *dummy_start = g_list_append (undo->priv->history_curr, action); // supress warning, we have left guard for start
+	GList *dummy_start = g_list_append (undo->priv->history_curr, action);
+
+	/* A dirty way to silence the compiler for these unused variables. */
+	(void) dummy_start;
+
 	undo->priv->history_curr = g_list_next (undo->priv->history_curr);
 
-	xpad_pad_notify_undo_redo_changed (xpad_text_buffer_get_pad (undo->priv->buffer));
+	g_object_get (undo->priv->buffer, "pad", &pad, NULL);
+	xpad_pad_notify_undo_redo_changed (pad);
 }
 
 void
@@ -397,6 +409,7 @@ xpad_undo_remove_tag (XpadUndo *undo, const gchar *name, GtkTextIter *start, Gtk
 	if (undo->priv->frozen)
 		return;
 
+	XpadPad *pad;
 	xpad_undo_clear_redo_history (undo);
 
 	gint start_offset = gtk_text_iter_get_offset (start);
@@ -409,10 +422,15 @@ xpad_undo_remove_tag (XpadUndo *undo, const gchar *name, GtkTextIter *start, Gtk
 	action->end = end_offset;
 	action->merged = FALSE;
 
-	GList *dummy_start = g_list_append (undo->priv->history_curr, action); // supress warning, we have left guard for start
+	GList *dummy_start = g_list_append (undo->priv->history_curr, action);
+
+	/* A dirty way to silence the compiler for these unused variables. */
+	(void) dummy_start;
+
 	undo->priv->history_curr = g_list_next (undo->priv->history_curr);
 
-	xpad_pad_notify_undo_redo_changed (xpad_text_buffer_get_pad (undo->priv->buffer));
+	g_object_get (undo->priv->buffer, "pad", &pad, NULL);
+	xpad_pad_notify_undo_redo_changed (pad);
 }
 
 gboolean
@@ -450,11 +468,8 @@ xpad_undo_redo_available (XpadUndo *undo)
 static void
 xpad_undo_get_start_end_iter (XpadUndo *undo, UserAction *action, GtkTextIter *start, GtkTextIter *end)
 {
-	gtk_text_buffer_get_iter_at_offset ( GTK_TEXT_BUFFER (undo->priv->buffer),
-			start, action->start);
-
-	gtk_text_buffer_get_iter_at_offset ( GTK_TEXT_BUFFER (undo->priv->buffer),
-			end, action->end);
+	gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (undo->priv->buffer), start, action->start);
+	gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (undo->priv->buffer), end, action->end);
 }
 
 void
@@ -463,9 +478,12 @@ xpad_undo_exec_undo (XpadUndo *undo)
 	if (!xpad_undo_undo_available (undo))
 		return;
 
+	XpadPad *pad;
+	g_object_get (undo->priv->buffer, "pad", &pad, NULL);
+
 	UserAction *action = undo->priv->history_curr->data;
 
-	GtkTextTagTable *table = gtk_text_buffer_get_tag_table ( GTK_TEXT_BUFFER (undo->priv->buffer));
+	GtkTextTagTable *table = gtk_text_buffer_get_tag_table (GTK_TEXT_BUFFER (undo->priv->buffer));
 
 	GtkTextIter start;
 	GtkTextIter end;
@@ -495,7 +513,7 @@ xpad_undo_exec_undo (XpadUndo *undo)
 						tag,
 						&start,
 						&end);
-				xpad_pad_save_content (xpad_text_buffer_get_pad (undo->priv->buffer));
+				xpad_pad_save_content (pad);
 			}
 			break;
 		case USER_ACTION_REMOVE_TAG:
@@ -505,14 +523,14 @@ xpad_undo_exec_undo (XpadUndo *undo)
 						tag,
 						&start,
 						&end);
-				xpad_pad_save_content (xpad_text_buffer_get_pad (undo->priv->buffer));
+				xpad_pad_save_content (pad);
 			}
 			break;
 	}
 
 	undo->priv->history_curr = g_list_previous (undo->priv->history_curr);
 
-	xpad_pad_notify_undo_redo_changed (xpad_text_buffer_get_pad (undo->priv->buffer));
+	xpad_pad_notify_undo_redo_changed (pad);
 }
 
 void
@@ -521,6 +539,9 @@ xpad_undo_exec_redo (XpadUndo *undo)
 	if (!xpad_undo_redo_available (undo))
 		return;
 
+	XpadPad *pad;
+	g_object_get (undo->priv->buffer, "pad", &pad, NULL);
+
 	UserAction *action = undo->priv->history_curr->next->data;
 
 	GtkTextTagTable *table = gtk_text_buffer_get_tag_table ( GTK_TEXT_BUFFER (undo->priv->buffer));
@@ -553,7 +574,7 @@ xpad_undo_exec_redo (XpadUndo *undo)
 						tag,
 						&start,
 						&end);
-				xpad_pad_save_content (xpad_text_buffer_get_pad (undo->priv->buffer));
+				xpad_pad_save_content (pad);
 			}
 			break;
 		case USER_ACTION_REMOVE_TAG:
@@ -563,14 +584,14 @@ xpad_undo_exec_redo (XpadUndo *undo)
 						tag,
 						&start,
 						&end);
-				xpad_pad_save_content (xpad_text_buffer_get_pad (undo->priv->buffer));
+				xpad_pad_save_content (pad);
 			}
 			break;
 	}
 
 	undo->priv->history_curr = g_list_next (undo->priv->history_curr);
 
-	xpad_pad_notify_undo_redo_changed (xpad_text_buffer_get_pad (undo->priv->buffer));
+	xpad_pad_notify_undo_redo_changed (pad);
 }
 
 void
